package eu.ztsh.wymiana.validation;

import eu.ztsh.wymiana.EntityCreator;
import eu.ztsh.wymiana.model.Symbol;
import eu.ztsh.wymiana.web.model.CurrencyExchangeRequest;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;

import java.math.BigDecimal;

import static eu.ztsh.wymiana.EntityCreator.Constants.USD_BUY;
import static eu.ztsh.wymiana.EntityCreator.Constants.USD_SELL;

class ValidExchangeRequestValidatorTest extends ValidatorTest<ValidExchangeRequestValidator, CurrencyExchangeRequest> {

    protected ValidExchangeRequestValidatorTest() {
        super(new ValidExchangeRequestValidator());
    }

    @Test
    @DisplayName("Valid request with buy value specified")
    void validRequestWithBuyTest() {
        assertThatValidation(EntityCreator.exchangeRequest()
            .from(Symbol.PLN)
            .to(Symbol.USD)
            .toBuy(USD_BUY)
            .build()).isTrue();
    }

    @Test
    @DisplayName("Valid request with sell value specified")
    void validRequestWithSellTest() {
        assertThatValidation(EntityCreator.exchangeRequest()
            .from(Symbol.PLN)
            .to(Symbol.USD)
            .toSell(USD_SELL)
            .build()).isTrue();
    }

    @Test
    @DisplayName("From and To have same value")
    void sameFromToTest() {
        assertThatValidation(EntityCreator.exchangeRequest()
            .from(Symbol.USD)
            .to(Symbol.USD)
            .toSell(USD_SELL)
            .build()).isFalse();
    }

    @Test
    @DisplayName("Empty amounts")
    void emptyBuySellTest() {
        assertThatValidation(EntityCreator.exchangeRequest()
            .from(Symbol.PLN)
            .to(Symbol.USD)
            .build()).isFalse();
    }

    @Test
    @DisplayName("Both Buy and Sell params filled in")
    void bothFilledBuySellTest() {
        assertThatValidation(EntityCreator.exchangeRequest()
            .from(Symbol.PLN)
            .to(Symbol.USD)
            .toBuy(USD_BUY)
            .toSell(USD_SELL)
            .build()).isFalse();
    }

    @Test
    @DisplayName("Negative buy amount value")
    void negativeBuyAmountTest() {
        assertThatValidation(EntityCreator.exchangeRequest()
            .from(Symbol.PLN)
            .to(Symbol.USD)
            .toBuy(BigDecimal.valueOf(-1.0))
            .build()).isFalse();
    }

    @Test
    @DisplayName("Negative sell amount value")
    void negativeSellAmountTest() {
        assertThatValidation(EntityCreator.exchangeRequest()
            .from(Symbol.PLN)
            .to(Symbol.USD)
            .toSell(BigDecimal.valueOf(-1.0))
            .build()).isFalse();
    }

}
